import base64
import os
from typing import Dict, Optional

import boto3  # type: ignore


def get_values_to_decrypt(variables: Dict[str, str]) -> Dict[str, str]:
    """Filters the environ pseudo-dict and identifies key/vals for decryption"""
    return {
        key[14:]: value
        for key, value in variables.items()
        if key.startswith("KMS_ENCRYPTED_")
    }


def decrypt_value(kms_client: boto3.client, value: str) -> Optional[str]:
    """Decrypt a single value"""
    try:
        decoded = base64.b64decode(value)
        result = kms_client.decrypt(CiphertextBlob=decoded)
        return result["Plaintext"].decode()
    except kms_client.exceptions.ClientError:
        return None


def decrypt(
    kms_client: Optional[boto3.client] = None,
    variables: Optional[Dict[str, str]] = None,
):
    """Decrypts encrypted keys in the environment"""
    if not kms_client:  # pragma: no cover
        kms_client = boto3.client("kms")

    if not variables:
        variables = os.environ  # type: ignore

    values = get_values_to_decrypt(variables)  # type: ignore
    for key, value in values.items():
        decrypted_value = decrypt_value(kms_client, value)
        if decrypted_value:
            os.environ[key] = decrypted_value



import os
from collections import namedtuple

import boto3
import pytest
from botocore.stub import Stubber

# prevent boto from looking for IAM creds via metadata while running tests
os.environ["AWS_EC2_METADATA_DISABLED"] = "true"
os.environ["AWS_DEFAULT_REGION"] = "ap-southeast-2"


StubbedClient = namedtuple("StubbedClient", ["stub", "client"])


@pytest.fixture
def fake_kms_client() -> StubbedClient:  # type: ignore
    """Creates a stubbed boto3 CloudFormation client"""
    client = boto3.client("kms")
    with Stubber(client) as stubbed_client:
        yield StubbedClient(stubbed_client, client)
        stubbed_client.assert_no_pending_responses()



KMS_KEY_ID = (
    "arn:aws:kms:ap-southeast-2:123456789012:key/aaaaaaaa-1111-aaaa-aaaa-aaaaaaaaaaaa"
)


def stub_decrypt(
    stubber,
    ciphertext_encoded: bytes,
    decrypted_value: str,
):
    """Stubs KMS decrypt responses"""
    response = {
        "KeyId": KMS_KEY_ID,
        "Plaintext": decrypted_value.encode(),
        "EncryptionAlgorithm": "SYMMETRIC_DEFAULT",
    }

    stubber.add_response(
        "decrypt",
        response,
        expected_params={"CiphertextBlob": ciphertext_encoded},
    )


def stub_decrypt_error(
    stubber,
    ciphertext_encoded: bytes,
    exception_type: str = "InvalidCiphertextException",
):
    """Stubs KMS decrypt responses where there is an error"""
    stubber.add_client_error(
        "decrypt",
        exception_type,
        exception_type,
        400,
        expected_params={"CiphertextBlob": ciphertext_encoded},
    )



# pylint:disable=redefined-outer-name
import base64
import os

from silenceofthelambs import main

from . import stubs
from .conftest import StubbedClient


def test_get_values_to_decrypt():
    """Tests main.get_values_to_decrypt()"""
    fake_environ = {"MY": "thing", "SOMETHING": "else"}
    assert dict() == main.get_values_to_decrypt(fake_environ)

    fake_environ = {
        "MY": "thing",
        "SOMETHING": "else",
        "KMS_ENCRYPTED_SOMETHING_ELSE": "sure",
    }
    assert {"SOMETHING_ELSE": "sure"} == main.get_values_to_decrypt(fake_environ)


def test_decrypt_value(fake_kms_client: StubbedClient):
    """Tests main.decrypt_value()"""
    value = "hello"
    encoded_value = base64.b64encode(value.encode()).decode()

    stubs.stub_decrypt(fake_kms_client.stub, value.encode(), value)
    assert value == main.decrypt_value(fake_kms_client.client, encoded_value)

    # test an invalid value
    stubs.stub_decrypt_error(fake_kms_client.stub, value.encode())
    assert not main.decrypt_value(fake_kms_client.client, encoded_value)


def test_decrypt(fake_kms_client: StubbedClient):
    """Tests main.decrypt()"""
    value = "hello"
    encoded_value = base64.b64encode(value.encode()).decode()
    os.environ.update(
        {
            "MY": "thing",
            "SOMETHING": "",
            "KMS_ENCRYPTED_SOMETHING": encoded_value,
        }
    )

    stubs.stub_decrypt(fake_kms_client.stub, value.encode(), "my_decoded_value")
    main.decrypt(kms_client=fake_kms_client.client)

    assert "SOMETHING" in os.environ
    assert os.environ.get("SOMETHING") == "my_decoded_value"

    first_value = "hello"
    encoded_first_value = base64.b64encode(first_value.encode()).decode()
    second_value = "my_value"
    encoded_second_value = base64.b64encode(second_value.encode()).decode()
    variables = {
        "MY": "thing",
        "SOMETHING": "",
        "SOMETHING_ELSE": "",
        "KMS_ENCRYPTED_SOMETHING": encoded_first_value,
        "KMS_ENCRYPTED_SOMETHING_ELSE": encoded_second_value,
    }

    stubs.stub_decrypt(fake_kms_client.stub, first_value.encode(), "my_decoded_value")
    stubs.stub_decrypt(
        fake_kms_client.stub, second_value.encode(), "another_decoded_value"
    )
    main.decrypt(kms_client=fake_kms_client.client, variables=variables)

    assert "SOMETHING" in os.environ
    assert os.environ.get("SOMETHING") == "my_decoded_value"
    assert "SOMETHING_ELSE" in os.environ
    assert os.environ.get("SOMETHING_ELSE") == "another_decoded_value"
